/*
 * Copyright (C) by Argonne National Laboratory
 *     See COPYRIGHT in top-level directory
 */

/* -- THIS FILE IS AUTO-GENERATED -- */

#include "mpiimpl.h"

/* -- Begin Profiling Symbol Block for routine MPIX_Session_get_nth_pset */
#if defined(HAVE_PRAGMA_WEAK)
#pragma weak MPIX_Session_get_nth_pset = PMPIX_Session_get_nth_pset
#elif defined(HAVE_PRAGMA_HP_SEC_DEF)
#pragma _HP_SECONDARY_DEF PMPIX_Session_get_nth_pset  MPIX_Session_get_nth_pset
#elif defined(HAVE_PRAGMA_CRI_DUP)
#pragma _CRI duplicate MPIX_Session_get_nth_pset as PMPIX_Session_get_nth_pset
#elif defined(HAVE_WEAK_ATTRIBUTE)
int MPIX_Session_get_nth_pset(MPIX_Session session, MPI_Info info, int n, int *pset_len,
                              char *pset_name)
                               __attribute__ ((weak, alias("PMPIX_Session_get_nth_pset")));
#endif
/* -- End Profiling Symbol Block */

/* Define MPICH_MPI_FROM_PMPI if weak symbols are not supported to build
   the MPI routines */
#ifndef MPICH_MPI_FROM_PMPI
#undef MPIX_Session_get_nth_pset
#define MPIX_Session_get_nth_pset PMPIX_Session_get_nth_pset

#endif

/*@
   MPIX_Session_get_nth_pset - Get the nth processes set

Input Parameters:
+ session - session (handle)
. info - info object (handle)
- n - index of the desired process set name (integer)

Input/Output Parameters:
. pset_len - length of the pset\_name argument (integer)

Output Parameters:
. pset_name - name of the nth process set (string)

.N ThreadSafe

.N Fortran

.N Errors
.N MPI_SUCCESS

.N MPI_ERR_ARG
.N MPI_ERR_INFO
.N MPI_ERR_SESSION
.N MPI_ERR_OTHER

@*/

int MPIX_Session_get_nth_pset(MPIX_Session session, MPI_Info info, int n, int *pset_len,
                              char *pset_name)
{
    int mpi_errno = MPI_SUCCESS;
    MPIR_Session *session_ptr = NULL;
    MPIR_Info *info_ptr = NULL;
    MPIR_FUNC_TERSE_STATE_DECL(MPID_STATE_MPIX_SESSION_GET_NTH_PSET);

    MPIR_ERRTEST_INITIALIZED_ORDIE();

    MPID_THREAD_CS_ENTER(GLOBAL, MPIR_THREAD_GLOBAL_ALLFUNC_MUTEX);
    MPIR_FUNC_TERSE_ENTER(MPID_STATE_MPIX_SESSION_GET_NTH_PSET);

#ifdef HAVE_ERROR_CHECKING
    {
        MPID_BEGIN_ERROR_CHECKS;
        {
            MPIR_ERRTEST_INFO_OR_NULL(info, mpi_errno);
        }
        MPID_END_ERROR_CHECKS;
    }
#endif /* HAVE_ERROR_CHECKING */

    MPIR_Session_get_ptr(session, session_ptr);
    if (info != MPI_INFO_NULL) {
        MPIR_Info_get_ptr(info, info_ptr);
    }

#ifdef HAVE_ERROR_CHECKING
    {
        MPID_BEGIN_ERROR_CHECKS;
        {
            MPIR_Session_valid_ptr(session_ptr, mpi_errno);
            if (mpi_errno) {
                goto fn_fail;
            }
            if (info != MPI_INFO_NULL) {
                MPIR_Info_valid_ptr(info_ptr, mpi_errno);
                if (mpi_errno) {
                    goto fn_fail;
                }
            }
            MPIR_ERRTEST_ARGNEG(n, "n", mpi_errno);
            MPIR_ERRTEST_ARGNULL(pset_len, "pset_len", mpi_errno);
            if (*pset_len) {
                MPIR_ERRTEST_ARGNULL(pset_name, "pset_name", mpi_errno);
            }
        }
        MPID_END_ERROR_CHECKS;
    }
#endif /* HAVE_ERROR_CHECKING */

    /* ... body of routine ... */
    mpi_errno = MPIR_Session_get_nth_pset_impl(session_ptr, info_ptr, n, pset_len, pset_name);
    if (mpi_errno) {
        goto fn_fail;
    }
    /* ... end of body of routine ... */

  fn_exit:
    MPIR_FUNC_TERSE_EXIT(MPID_STATE_MPIX_SESSION_GET_NTH_PSET);
    MPID_THREAD_CS_EXIT(GLOBAL, MPIR_THREAD_GLOBAL_ALLFUNC_MUTEX);
    return mpi_errno;

  fn_fail:
    /* --BEGIN ERROR HANDLINE-- */
#ifdef HAVE_ERROR_CHECKING
    mpi_errno = MPIR_Err_create_code(mpi_errno, MPIR_ERR_RECOVERABLE, __func__, __LINE__, MPI_ERR_OTHER,
                                     "**mpix_session_get_nth_pset",
                                     "**mpix_session_get_nth_pset %S %I %d %p %p", session, info, n,
                                     pset_len, pset_name);
#endif
    mpi_errno = MPIR_Err_return_comm(0, __func__, mpi_errno);
    /* --END ERROR HANDLING-- */
    goto fn_exit;
}
