/*
 * Copyright (C) by Argonne National Laboratory
 *     See COPYRIGHT in top-level directory
 */

/* -- THIS FILE IS AUTO-GENERATED -- */

#include "mpiimpl.h"

/* -- Begin Profiling Symbol Block for routine MPIX_Session_init */
#if defined(HAVE_PRAGMA_WEAK)
#pragma weak MPIX_Session_init = PMPIX_Session_init
#elif defined(HAVE_PRAGMA_HP_SEC_DEF)
#pragma _HP_SECONDARY_DEF PMPIX_Session_init  MPIX_Session_init
#elif defined(HAVE_PRAGMA_CRI_DUP)
#pragma _CRI duplicate MPIX_Session_init as PMPIX_Session_init
#elif defined(HAVE_WEAK_ATTRIBUTE)
int MPIX_Session_init(MPI_Info info, MPI_Errhandler errhandler, MPIX_Session *session)
     __attribute__ ((weak, alias("PMPIX_Session_init")));
#endif
/* -- End Profiling Symbol Block */

/* Define MPICH_MPI_FROM_PMPI if weak symbols are not supported to build
   the MPI routines */
#ifndef MPICH_MPI_FROM_PMPI
#undef MPIX_Session_init
#define MPIX_Session_init PMPIX_Session_init

#endif

/*@
   MPIX_Session_init - Initialize an MPI session

Input Parameters:
+ info - info object to specify thread support level and MPI implementation specific resources (handle)
- errhandler - error handler to invoke in the event that an error is encountered during this function call (handle)

Output Parameters:
. session - new session (handle)

.N ThreadSafe

.N Fortran

.N Errors
.N MPI_SUCCESS

.N MPI_ERR_ARG
.N MPI_ERR_INFO
.N MPI_ERR_OTHER

@*/

int MPIX_Session_init(MPI_Info info, MPI_Errhandler errhandler, MPIX_Session *session)
{
    int mpi_errno = MPI_SUCCESS;
    MPIR_Info *info_ptr = NULL;
    MPIR_Errhandler *errhandler_ptr = NULL;
    MPIR_FUNC_TERSE_STATE_DECL(MPID_STATE_MPIX_SESSION_INIT);
    MPIR_FUNC_TERSE_ENTER(MPID_STATE_MPIX_SESSION_INIT);

#ifdef HAVE_ERROR_CHECKING
    {
        MPID_BEGIN_ERROR_CHECKS;
        {
            MPIR_ERRTEST_INFO_OR_NULL(info, mpi_errno);
            MPIR_ERRTEST_ERRHANDLER(errhandler, mpi_errno);
        }
        MPID_END_ERROR_CHECKS;
    }
#endif /* HAVE_ERROR_CHECKING */

    if (info != MPI_INFO_NULL) {
        MPIR_Info_get_ptr(info, info_ptr);
    }
    MPIR_Errhandler_get_ptr(errhandler, errhandler_ptr);

#ifdef HAVE_ERROR_CHECKING
    {
        MPID_BEGIN_ERROR_CHECKS;
        {
            if (info != MPI_INFO_NULL) {
                MPIR_Info_valid_ptr(info_ptr, mpi_errno);
                if (mpi_errno) {
                    goto fn_fail;
                }
            }
            MPIR_Errhandler_valid_ptr(errhandler_ptr, mpi_errno);
            if (mpi_errno) {
                goto fn_fail;
            }
            MPIR_ERRTEST_ARGNULL(session, "session", mpi_errno);
        }
        MPID_END_ERROR_CHECKS;
    }
#endif /* HAVE_ERROR_CHECKING */

    /* ... body of routine ... */
    MPIR_Session *session_ptr = NULL;
    *session = MPI_SESSION_NULL;
    mpi_errno = MPIR_Session_init_impl(info_ptr, errhandler_ptr, &session_ptr);
    if (mpi_errno) {
        goto fn_fail;
    }
    if (session_ptr) {
        MPIR_OBJ_PUBLISH_HANDLE(*session, session_ptr->handle);
    }
    /* ... end of body of routine ... */

  fn_exit:
    MPIR_FUNC_TERSE_EXIT(MPID_STATE_MPIX_SESSION_INIT);
    return mpi_errno;

  fn_fail:
    /* --BEGIN ERROR HANDLINE-- */
#ifdef HAVE_ERROR_CHECKING
    mpi_errno = MPIR_Err_create_code(mpi_errno, MPIR_ERR_RECOVERABLE, __func__, __LINE__, MPI_ERR_OTHER,
                                     "**mpix_session_init", "**mpix_session_init %I %E %p", info,
                                     errhandler, session);
#endif
    mpi_errno = MPIR_Err_return_comm(0, __func__, mpi_errno);
    /* --END ERROR HANDLING-- */
    goto fn_exit;
}
