/*
 * Copyright (C) by Argonne National Laboratory
 *     See COPYRIGHT in top-level directory
 */

/* -- THIS FILE IS AUTO-GENERATED -- */

#include "mpiimpl.h"

/* -- Begin Profiling Symbol Block for routine MPIX_T_event_get_info */
#if defined(HAVE_PRAGMA_WEAK)
#pragma weak MPIX_T_event_get_info = PMPIX_T_event_get_info
#elif defined(HAVE_PRAGMA_HP_SEC_DEF)
#pragma _HP_SECONDARY_DEF PMPIX_T_event_get_info  MPIX_T_event_get_info
#elif defined(HAVE_PRAGMA_CRI_DUP)
#pragma _CRI duplicate MPIX_T_event_get_info as PMPIX_T_event_get_info
#elif defined(HAVE_WEAK_ATTRIBUTE)
int MPIX_T_event_get_info(int event_index, char *name, int *name_len, int *verbosity,
                          MPI_Datatype array_of_datatypes[], MPI_Aint array_of_displacements[],
                          int *num_elements, MPI_T_enum *enumtype, MPI_Info *info, char *desc,
                          int *desc_len, int *bind)
                           __attribute__ ((weak, alias("PMPIX_T_event_get_info")));
#endif
/* -- End Profiling Symbol Block */

/* Define MPICH_MPI_FROM_PMPI if weak symbols are not supported to build
   the MPI routines */
#ifndef MPICH_MPI_FROM_PMPI
#undef MPIX_T_event_get_info
#define MPIX_T_event_get_info PMPIX_T_event_get_info

#endif

/*@
   MPIX_T_event_get_info - Returns additional information about a specific event type

Input Parameters:
. event_index - index of the event type to be queried between $0$ and $num_events-1$ (integer)

Input/Output Parameters:
+ name_len - length of the string and/or buffer for name (integer)
. num_elements - length of array\\_of\\_datatypes and array\\_of\\_displacements arrays (non-negative integer)
- desc_len - length of the string and/or buffer for desc (integer)

Output Parameters:
+ name - buffer to return the string containing the name of the event type (string)
. verbosity - verbosity level of this event type (integer)
. array_of_datatypes - array of mpi basic datatypes used to encode the event data (handle)
. array_of_displacements - array of byte displacements of the elements in the event buffer (non-negative integer)
. enumtype - optional descriptor for enumeration information (handle)
. info - optional info object (handle)
. desc - buffer to return the string containing a description of the event type (string)
- bind - type of mpi object to which an event of this type must be bound (integer)

.N ThreadSafe

.N Errors
.N MPI_SUCCESS

.N MPI_T_ERR_INVALID
.N MPI_T_ERR_INVALID_INDEX
.N MPI_T_ERR_NOT_INITIALIZED
.N MPI_ERR_OTHER

@*/

int MPIX_T_event_get_info(int event_index, char *name, int *name_len, int *verbosity,
                          MPI_Datatype array_of_datatypes[], MPI_Aint array_of_displacements[],
                          int *num_elements, MPI_T_enum *enumtype, MPI_Info *info, char *desc,
                          int *desc_len, int *bind)
{
    int mpi_errno = MPI_SUCCESS;
    MPIR_FUNC_TERSE_STATE_DECL(MPID_STATE_MPIX_T_EVENT_GET_INFO);

    MPIT_ERRTEST_MPIT_INITIALIZED();

    MPIR_T_THREAD_CS_ENTER();
    MPIR_FUNC_TERSE_ENTER(MPID_STATE_MPIX_T_EVENT_GET_INFO);

#ifdef HAVE_ERROR_CHECKING
    {
        MPID_BEGIN_ERROR_CHECKS;
        {
            MPIT_ERRTEST_ARGNULL(name_len);
            MPIT_ERRTEST_ARGNULL(verbosity);
            MPIT_ERRTEST_ARGNULL(enumtype);
            MPIT_ERRTEST_ARGNULL(desc_len);
            MPIT_ERRTEST_ARGNULL(bind);
        }
        MPID_END_ERROR_CHECKS;
    }
#endif /* HAVE_ERROR_CHECKING */

    /* ... body of routine ... */
    mpi_errno = MPIR_T_event_get_info_impl(event_index, name, name_len, verbosity, array_of_datatypes,
                                           array_of_displacements, num_elements, enumtype, info, desc,
                                           desc_len, bind);
    if (mpi_errno) {
        goto fn_fail;
    }
    /* ... end of body of routine ... */

  fn_exit:
    MPIR_FUNC_TERSE_EXIT(MPID_STATE_MPIX_T_EVENT_GET_INFO);
    MPIR_T_THREAD_CS_EXIT();
    return mpi_errno;

  fn_fail:
    goto fn_exit;
}
